@@ -7,14 +7,14 @@
 from glob import glob
 from pathlib import Path
 from time import time, sleep
-from typing import Union, List, Any
+from typing import Union, List, Any, Tuple
 from urllib.parse import quote
 
 from selenium.common.exceptions import NoAlertPresentException
 from selenium.webdriver.chrome.webdriver import WebDriver
 from selenium.webdriver.remote.webelement import WebElement
 
-from .common import get_loc_from_str, get_available_file_name
+from .common import str_to_loc, get_available_file_name, translate_loc, format_html
 from .driver_element import DriverElement, execute_driver_find
 
 
@@ -43,7 +43,7 @@ class DriverPage(object):
     @property
     def html(self) -> str:
         """返回页面html文本"""
-        return self.driver.find_element_by_xpath("//*").get_attribute("outerHTML")
+        return format_html(self.driver.find_element_by_xpath("//*").get_attribute("outerHTML"))
 
     @property
     def url_available(self) -> bool:
@@ -60,11 +60,11 @@ class DriverPage(object):
         """返回网页title"""
         return self.driver.title
 
-    def _try_to_get(self,
-                    to_url: str,
-                    times: int = 0,
-                    interval: float = 1,
-                    show_errmsg: bool = False, ):
+    def _try_to_connect(self,
+                        to_url: str,
+                        times: int = 0,
+                        interval: float = 1,
+                        show_errmsg: bool = False, ):
         """尝试连接，重试若干次                            \n
         :param to_url: 要访问的url
         :param times: 重试次数
@@ -102,14 +102,13 @@ class DriverPage(object):
         if not url or (not go_anyway and self.url == to_url):
             return
         self._url = to_url
-        self._url_available = self._try_to_get(to_url, times=retry, interval=interval, show_errmsg=show_errmsg)
+        self._url_available = self._try_to_connect(to_url, times=retry, interval=interval, show_errmsg=show_errmsg)
         return self._url_available
 
     def ele(self,
-            loc_or_ele: Union[tuple, str, DriverElement, WebElement],
+            loc_or_ele: Union[Tuple[str, str], str, DriverElement, WebElement],
             mode: str = None,
-            timeout: float = None,
-            show_errmsg: bool = False) -> Union[DriverElement, List[DriverElement], None]:
+            timeout: float = None) -> Union[DriverElement, List[DriverElement or str], str, None]:
         """返回页面中符合条件的元素，默认返回第一个                                                         \n
         示例：                                                                                           \n
         - 接收到元素对象时：                                                                              \n
@@ -135,27 +134,38 @@ class DriverPage(object):
         :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
         :param mode: 'single' 或 'all‘，对应查找一个或全部
         :param timeout: 查找元素超时时间
-        :param show_errmsg: 出现异常时是否打印信息
         :return: DriverElement对象
         """
-        if isinstance(loc_or_ele, str):
-            loc_or_ele = get_loc_from_str(loc_or_ele)
-            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith('/'):
-                loc_or_ele = 'xpath', f'//{loc_or_ele[1]}'
-        elif isinstance(loc_or_ele, tuple) and len(loc_or_ele) == 2:
-            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith('/'):
-                loc_or_ele = 'xpath', f'//{loc_or_ele[1]}'
+        # 接收到字符串或元组，获取定位loc元组
+        if isinstance(loc_or_ele, (str, tuple)):
+            if isinstance(loc_or_ele, str):
+                loc_or_ele = str_to_loc(loc_or_ele)
+            else:
+                if len(loc_or_ele) != 2:
+                    raise ValueError("Len of loc_or_ele must be 2 when it's a tuple.")
+                loc_or_ele = translate_loc(loc_or_ele)
+
+            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith(('/', '(')):
+                loc_or_ele = loc_or_ele[0], f'//{loc_or_ele[1]}'
+
+        # 接收到DriverElement对象直接返回
         elif isinstance(loc_or_ele, DriverElement):
             return loc_or_ele
+
+        # 接收到WebElement对象打包成DriverElement对象返回
         elif isinstance(loc_or_ele, WebElement):
-            return DriverElement(loc_or_ele, self.timeout)
+            return DriverElement(loc_or_ele, self, self.timeout)
+
+        # 接收到的类型不正确，抛出异常
         else:
             raise ValueError('Argument loc_or_str can only be tuple, str, DriverElement, DriverElement.')
 
         timeout = timeout or self.timeout
-        return execute_driver_find(self.driver, loc_or_ele, mode, show_errmsg, timeout)
+        return execute_driver_find(self, loc_or_ele, mode, timeout)
 
-    def eles(self, loc_or_str: Union[tuple, str], timeout: float = None, show_errmsg=False) -> List[DriverElement]:
+    def eles(self,
+             loc_or_str: Union[Tuple[str, str], str],
+             timeout: float = None) -> List[DriverElement or str]:
         """返回页面中所有符合条件的元素                                                                     \n
         示例：                                                                                            \n
         - 用loc元组查找：                                                                                 \n
@@ -178,12 +188,11 @@ class DriverPage(object):
             page.eles('css:div.ele_class')                - 返回所有符合css selector的元素                 \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
         :param timeout: 查找元素超时时间
-        :param show_errmsg: 出现异常时是否打印信息
         :return: DriverElement对象组成的列表
         """
         if not isinstance(loc_or_str, (tuple, str)):
             raise TypeError('Type of loc_or_str can only be tuple or str.')
-        return self.ele(loc_or_str, mode='all', timeout=timeout, show_errmsg=show_errmsg)
+        return self.ele(loc_or_str, mode='all', timeout=timeout)
 
     # ----------------以下为独有函数-----------------------
     def wait_ele(self,
@@ -210,7 +219,7 @@ class DriverPage(object):
         elif isinstance(loc_or_ele, WebElement):
             is_ele = True
         elif isinstance(loc_or_ele, str):
-            loc_or_ele = get_loc_from_str(loc_or_ele)
+            loc_or_ele = str_to_loc(loc_or_ele)
         elif isinstance(loc_or_ele, tuple):
             pass
         else:
@@ -293,7 +302,7 @@ class DriverPage(object):
         if self.tabs_count:
             self.to_tab(0)
 
-    def close_other_tabs(self, num_or_handle: Union[int, str, None] = None) -> None:
+    def close_other_tabs(self, num_or_handle: Union[int, str] = None) -> None:
         """关闭传入的标签页以外标签页，默认保留当前页                                \n
         :param num_or_handle: 要保留的标签页序号或handle，序号第一个为0，最后为-1
         :return: None
