@@ -4,26 +4,27 @@
 @Contact :   g1879@qq.com
 @File    :   session_page.py
 """
+import re
 from os import path as os_PATH
 from pathlib import Path
 from random import randint
 from re import search as re_SEARCH
 from re import sub as re_SUB
 from time import time, sleep
-from typing import Union, List
-from urllib.parse import urlparse, quote
+from typing import Union, List, Tuple
+from urllib.parse import urlparse, quote, unquote
 
-from requests_html import HTMLSession, HTMLResponse, Element
+from requests import Session, Response
 
-from .common import get_loc_from_str, translate_loc_to_xpath, get_available_file_name
+from .common import str_to_loc, translate_loc, get_available_file_name, format_html
 from .config import OptionsManager
 from .session_element import SessionElement, execute_session_find
 
 
 class SessionPage(object):
-    """SessionPage封装了页面操作的常用功能，使用requests_html来获取、解析网页。"""
+    """SessionPage封装了页面操作的常用功能，使用requests来获取、解析网页"""
 
-    def __init__(self, session: HTMLSession, timeout: float = 10):
+    def __init__(self, session: Session, timeout: float = 10):
         """初始化函数"""
         self._session = session
         self.timeout = timeout
@@ -32,12 +33,12 @@ class SessionPage(object):
         self._response = None
 
     @property
-    def session(self) -> HTMLSession:
+    def session(self) -> Session:
         """返回session对象"""
         return self._session
 
     @property
-    def response(self) -> HTMLResponse:
+    def response(self) -> Response:
         """返回访问url得到的response对象"""
         return self._response
 
@@ -64,13 +65,12 @@ class SessionPage(object):
     @property
     def html(self) -> str:
         """返回页面html文本"""
-        return self.response.html.html
+        return format_html(self.response.text)
 
     def ele(self,
-            loc_or_ele: Union[tuple, str, SessionElement, Element],
-            mode: str = None,
-            show_errmsg: bool = False) -> Union[SessionElement, List[SessionElement], None]:
-        """返回页面中符合条件的元素，默认返回第一个                                                          \n
+            loc_or_ele: Union[Tuple[str, str], str, SessionElement],
+            mode: str = None) -> Union[SessionElement, List[SessionElement or str], str, None]:
+        """返回页面中符合条件的元素、属性或节点文本，默认返回第一个                                           \n
         示例：                                                                                           \n
         - 接收到元素对象时：                                                                              \n
             返回SessionElement对象                                                                        \n
@@ -78,7 +78,7 @@ class SessionPage(object):
             ele.ele((By.CLASS_NAME, 'ele_class')) - 返回所有class为ele_class的子元素                       \n
         - 用查询字符串查找：                                                                               \n
             查找方式：属性、tag name和属性、文本、xpath、css selector                                        \n
-            其中，@表示属性，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串                          \n
+            其中，@表示属性，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串                        \n
             page.ele('@class:ele_class')                 - 返回第一个class含有ele_class的元素              \n
             page.ele('@name=ele_name')                   - 返回第一个name等于ele_name的元素                \n
             page.ele('@placeholder')                     - 返回第一个带placeholder属性的元素               \n
@@ -94,33 +94,36 @@ class SessionPage(object):
             page.ele('css:div.ele_class')                - 返回第一个符合css selector的元素                 \n
         :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
         :param mode: 'single' 或 'all‘，对应查找一个或全部
-        :param show_errmsg: 出现异常时是否打印信息
         :return: SessionElement对象
         """
-        if isinstance(loc_or_ele, str):
-            loc_or_ele = get_loc_from_str(loc_or_ele)
-            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith('/'):
-                loc_or_ele = 'xpath', f'//{loc_or_ele[1]}'
-        elif isinstance(loc_or_ele, tuple) and len(loc_or_ele) == 2:
-            loc_or_ele = translate_loc_to_xpath(loc_or_ele)
-            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith('/'):
-                loc_or_ele = 'xpath', f'//{loc_or_ele[1]}'
+        if isinstance(loc_or_ele, (str, tuple)):
+            if isinstance(loc_or_ele, str):
+                loc_or_ele = str_to_loc(loc_or_ele)
+            else:
+                if len(loc_or_ele) != 2:
+                    raise ValueError("Len of loc_or_ele must be 2 when it's a tuple.")
+                loc_or_ele = translate_loc(loc_or_ele)
+
+            if loc_or_ele[0] == 'xpath' and not loc_or_ele[1].startswith(('/', '(')):
+                loc_or_ele = loc_or_ele[0], f'//{loc_or_ele[1]}'
+
         elif isinstance(loc_or_ele, SessionElement):
             return loc_or_ele
-        elif isinstance(loc_or_ele, Element):
-            return SessionElement(loc_or_ele)
+
         else:
             raise ValueError('Argument loc_or_str can only be tuple, str, SessionElement, Element.')
-        return execute_session_find(self.response.html, loc_or_ele, mode, show_errmsg)
 
-    def eles(self, loc_or_str: Union[tuple, str], show_errmsg: bool = False) -> List[SessionElement]:
-        """返回页面中所有符合条件的元素                                                                    \n
+        return execute_session_find(self, loc_or_ele, mode)
+
+    def eles(self,
+             loc_or_str: Union[Tuple[str, str], str]) -> List[SessionElement or str]:
+        """返回页面中所有符合条件的元素、属性或节点文本                                                     \n
         示例：                                                                                          \n
         - 用loc元组查找：                                                                                \n
             page.eles((By.CLASS_NAME, 'ele_class')) - 返回所有class为ele_class的元素                     \n
         - 用查询字符串查找：                                                                              \n
             查找方式：属性、tag name和属性、文本、xpath、css selector                                       \n
-            其中，@表示属性，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串                         \n
+            其中，@表示属性，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串                       \n
             page.eles('@class:ele_class')                 - 返回所有class含有ele_class的元素              \n
             page.eles('@name=ele_name')                   - 返回所有name等于ele_name的元素                \n
             page.eles('@placeholder')                     - 返回所有带placeholder属性的元素               \n
@@ -135,34 +138,37 @@ class SessionPage(object):
             page.eles('xpath://div[@class="ele_class"]')  - 返回所有符合xpath的元素                        \n
             page.eles('css:div.ele_class')                - 返回所有符合css selector的元素                 \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param show_errmsg: 出现异常时是否打印信息
         :return: SessionElement对象组成的列表
         """
         if not isinstance(loc_or_str, (tuple, str)):
             raise TypeError('Type of loc_or_str can only be tuple or str.')
-        return self.ele(loc_or_str, mode='all', show_errmsg=True)
+        return self.ele(loc_or_str, mode='all')
 
-    def _try_to_get(self,
-                    to_url: str,
-                    times: int = 0,
-                    interval: float = 1,
-                    show_errmsg: bool = False,
-                    **kwargs) -> HTMLResponse:
+    def _try_to_connect(self,
+                        to_url: str,
+                        times: int = 0,
+                        interval: float = 1,
+                        mode: str = 'get',
+                        data: dict = None,
+                        show_errmsg: bool = False,
+                        **kwargs) -> Response:
         """尝试连接，重试若干次                            \n
         :param to_url: 要访问的url
         :param times: 重试次数
         :param interval: 重试间隔（秒）
+        :param mode: 连接方式，'get' 或 'post'
+        :param data: post方式提交的数据
         :param show_errmsg: 是否抛出异常
         :param kwargs: 连接参数
         :return: HTMLResponse对象
         """
-        r = self._make_response(to_url, show_errmsg=show_errmsg, **kwargs)[0]
+        r = self._make_response(to_url, mode=mode, show_errmsg=show_errmsg, **kwargs)[0]
         while times and (not r or r.content == b''):
             if r is not None and r.status_code in (403, 404):
                 break
             print('重试', to_url)
             sleep(interval)
-            r = self._make_response(to_url, show_errmsg=show_errmsg, **kwargs)[0]
+            r = self._make_response(to_url, mode=mode, show_errmsg=show_errmsg, **kwargs)[0]
             times -= 1
         return r
 
@@ -183,26 +189,23 @@ class SessionPage(object):
         :return: url是否可用
         """
         to_url = quote(url, safe='/:&?=%;#@+')
+
         if not url or (not go_anyway and self.url == to_url):
             return
+
         self._url = to_url
-        self._response = self._try_to_get(to_url, times=retry, interval=interval, show_errmsg=show_errmsg, **kwargs)
+        self._response = self._try_to_connect(to_url, times=retry, interval=interval, show_errmsg=show_errmsg, **kwargs)
+
         if self._response is None:
             self._url_available = False
         else:
-            stream = tuple(x for x in kwargs if x.lower() == 'stream')
-            if (not stream or not kwargs[stream[0]]) and not self.session.stream:
-                try:
-                    self._response.html.encoding = self._response.encoding  # 修复requests_html丢失编码方式的bug
-                except:
-                    pass
-
             if self._response.ok:
                 self._url_available = True
             else:
                 if show_errmsg:
-                    raise ConnectionError(f'Status code: {self._response.status_code}.')
+                    raise ConnectionError(f'{to_url}\nStatus code: {self._response.status_code}.')
                 self._url_available = False
+
         return self._url_available
 
     def post(self,
@@ -210,34 +213,37 @@ class SessionPage(object):
              data: dict = None,
              go_anyway: bool = True,
              show_errmsg: bool = False,
+             retry: int = 0,
+             interval: float = 1,
              **kwargs) -> Union[bool, None]:
         """用post方式跳转到url                                 \n
         :param url: 目标url
         :param data: 提交的数据
         :param go_anyway: 若目标url与当前url一致，是否强制跳转
         :param show_errmsg: 是否显示和抛出异常
+        :param retry: 重试次数
+        :param interval: 重试间隔（秒）
         :param kwargs: 连接参数
         :return: url是否可用
         """
         to_url = quote(url, safe='/:&?=%;#@')
+
         if not url or (not go_anyway and self._url == to_url):
             return
+
         self._url = to_url
-        self._response = self._make_response(to_url, mode='post', data=data, show_errmsg=show_errmsg, **kwargs)[0]
+        self._response = self._try_to_connect(to_url, retry, interval, 'post', data, show_errmsg, **kwargs)
+
         if self._response is None:
             self._url_available = False
         else:
-            try:
-                self._response.html.encoding = self._response.encoding  # 修复requests_html丢失编码方式的bug
-            except:
-                pass
-
             if self._response.ok:
                 self._url_available = True
             else:
                 if show_errmsg:
                     raise ConnectionError(f'Status code: {self._response.status_code}.')
                 self._url_available = False
+
         return self._url_available
 
     def download(self,
@@ -245,6 +251,7 @@ class SessionPage(object):
                  goal_path: str = None,
                  rename: str = None,
                  file_exists: str = 'rename',
+                 post_data: dict = None,
                  show_msg: bool = False,
                  show_errmsg: bool = False,
                  **kwargs) -> tuple:
@@ -253,6 +260,7 @@ class SessionPage(object):
         :param goal_path: 存放路径
         :param rename: 重命名文件，可不写扩展名
         :param file_exists: 若存在同名文件，可选择 'rename', 'overwrite', 'skip' 方式处理
+        :param post_data: post方式的数据
         :param show_msg: 是否显示下载信息
         :param show_errmsg: 是否抛出和显示异常
         :param kwargs: 连接参数
@@ -267,23 +275,42 @@ class SessionPage(object):
         if 'timeout' not in kwargs:
             kwargs['timeout'] = 20
 
-        r, info = self._make_response(file_url, mode='get', show_errmsg=show_errmsg, **kwargs)
+        if not post_data:
+            r, info = self._make_response(file_url, mode='get', show_errmsg=show_errmsg, **kwargs)
+        else:
+            r, info = self._make_response(file_url, mode='post', data=post_data, show_errmsg=show_errmsg, **kwargs)
+
         if r is None:
             if show_msg:
                 print(info)
             return False, info
+
         if not r.ok:
             if show_errmsg:
                 raise ConnectionError(f'Status code: {r.status_code}.')
             return False, f'Status code: {r.status_code}.'
+
         # -------------------获取文件名-------------------
-        if 'Content-disposition' in r.headers:  # header里有文件名，则使用它
-            file_name = r.headers['Content-disposition'].split('"')[1].encode('ISO-8859-1').decode('utf-8')
-        elif os_PATH.basename(file_url):  # 在url里获取文件名
+        file_name = ''
+        content_disposition = tuple(x for x in r.headers if x.lower() == 'content-disposition')
+
+        # header里有文件名，则使用它
+        if content_disposition:
+            file_name = r.headers[content_disposition[0]].encode('ISO-8859-1').decode('utf-8')
+            file_name = re.search(r'filename *= *"?([^";]+)', file_name)
+            if file_name:
+                file_name = file_name.group(1)
+            if file_name[0] == file_name[-1] == "'":
+                file_name = file_name.strip("'")
+
+        if not file_name and os_PATH.basename(file_url):  # 在url里获取文件名
             file_name = os_PATH.basename(file_url).split("?")[0]
-        else:  # 找不到则用时间和随机数生成文件名
+
+        if not file_name:  # 找不到则用时间和随机数生成文件名
             file_name = f'untitled_{time()}_{randint(0, 100)}'
         file_name = re_SUB(r'[\\/*:|<>?"]', '', file_name).strip()  # 去除非法字符
+        file_name = unquote(file_name)
+
         # -------------------重命名文件名-------------------
         if rename:  # 重命名文件，不改变扩展名
             rename = re_SUB(r'[\\/*:|<>?"]', '', rename).strip()
@@ -294,6 +321,7 @@ class SessionPage(object):
                 full_name = f'{rename}.{ext_name}'
         else:
             full_name = file_name
+
         # -------------------生成路径-------------------
         goal_Path = Path(goal_path)
         goal_path = ''
@@ -315,6 +343,7 @@ class SessionPage(object):
                 full_path = Path(f'{goal_path}\\{full_name}')
             else:
                 raise ValueError("Argument file_exists can only be 'skip', 'overwrite', 'rename'.")
+
         # -------------------打印要下载的文件-------------------
         if show_msg:
             print(full_name if file_name == full_name else f'{file_name} -> {full_name}')
@@ -322,7 +351,8 @@ class SessionPage(object):
 
         # -------------------开始下载-------------------
         # 获取远程文件大小
-        file_size = int(r.headers['Content-Length']) if 'Content-Length' in r.headers else None
+        content_length = tuple(x for x in r.headers if x.lower() == 'content-length')
+        file_size = int(r.headers[content_length]) if content_length else None
         downloaded_size, download_status = 0, False  # 已下载文件大小和下载状态
         try:
             with open(str(full_path), 'wb') as tmpFile:
@@ -348,6 +378,7 @@ class SessionPage(object):
             if not download_status and full_path.exists():
                 full_path.unlink()  # 删除下载出错文件
             r.close()
+
         # -------------------显示并返回值-------------------
         if show_msg:
             print(info)
@@ -395,33 +426,33 @@ class SessionPage(object):
                 r = self.session.get(url, **kwargs)
             elif mode == 'post':
                 r = self.session.post(url, data=data, **kwargs)
+
         except Exception as e:
             if show_errmsg:
                 raise e
             return None, e
+
         else:
             headers = dict(r.headers)
             content_type = tuple(x for x in headers if x.lower() == 'content-type')
             stream = tuple(x for x in kwargs if x.lower() == 'stream')
+            not_stream = (not stream or not kwargs[stream[0]]) and not self.session.stream
             charset = None
             if not content_type or 'charset' not in headers[content_type[0]].lower():
-                if (not stream or not kwargs[stream[0]]) and not self.session.stream:
-                    # ========================
-                    re_result = None
-                    for chunk in r.iter_content(chunk_size=512):
-                        re_result = re_SEARCH(r'<meta.*?charset=[ \'"]*([^"\' />]+).*?>', chunk.decode())
-                        break
-                    # ========================
-                    # re_result = re_SEARCH(r'<meta.*?charset=[ \'"]*([^"\' />]+).*?>', r.text)
+                if not_stream:
+                    re_result = re_SEARCH(r'<meta.*?charset=[ \'"]*([^"\' />]+).*?>',
+                                          r.iter_content(chunk_size=512).__next__().decode())
                     try:
                         charset = re_result.group(1)
                     except:
                         charset = r.apparent_encoding
             else:
                 charset = headers[content_type[0]].split('=')[1]
-            # 避免存在退格符导致乱码或解析出错
-            if (not stream or not kwargs[stream[0]]) and not self.session.stream:
-                r._content = r.content.replace(b'\x08', b'\\b')
-            if charset:
+
+            if charset:  # 指定网页编码
                 r.encoding = charset
+
+            if not_stream:  # 避免存在退格符导致乱码或解析出错
+                r._content = r.content.replace(b'\x08', b'\\b')
+
             return r, 'Success'
